import useWindowDimensions from "hooks/useWindowDimensions";
import Image from "next/image";
import { Media, Page } from "payload-types";
import { useEffect, useRef, useState } from "react";

import { createImageUrl } from "utils/helper";

import { Heading } from "components/Heading";

const delay = 5000;

type OwlCarouselBlock = Extract<
  Extract<Page["layout"][number], { blockType: "container" }>["layout"][number],
  { blockType: "owlCarousel" }
> & { image: Media };

const OwlCarousel = ({ items }: OwlCarouselBlock) => {
  const [index, setIndex] = useState(0);
  const timeoutRef = useRef(null);
  const { width } = useWindowDimensions();
  const [transformValue, setTransformValue] = useState(0);
  const [touchPosition, setTouchPosition] = useState(null);

  const resetTimeout = () => {
    if (timeoutRef.current) {
      clearTimeout(timeoutRef.current);
    }
  };

  const handleTouchStart = (e) => {
    const touchDown = e.touches[0].clientX;
    setTouchPosition(touchDown);
  };

  const handleTouchMove = (e) => {
    const touchDown = touchPosition;

    if (touchDown === null) {
      return;
    }

    const currentTouch = e.touches[0].clientX;
    const diff = touchDown - currentTouch;

    if (diff > 5) {
      setIndex((prevIndex) => (prevIndex === items.length - 1 ? 0 : prevIndex + 1));
    }

    if (diff < -5) {
      setIndex((prevIndex) => (prevIndex === 0 ? items.length - 1 : prevIndex - 1));
    }

    setTouchPosition(null);
  };

  useEffect(() => {
    setTransformValue(width > 768 ? 50 : 100);
  }, [width]);

  useEffect(() => {
    resetTimeout();
    timeoutRef.current = setTimeout(
      () => setIndex((prevIndex) => (prevIndex === items.length - 1 ? 0 : prevIndex + 1)),
      delay
    );

    return () => {
      resetTimeout();
    };
  }, [index, items]);

  return (
    <div
      className="w-full flex flex-col items-center overflow-hidden"
      onTouchStart={handleTouchStart}
      onTouchMove={handleTouchMove}
    >
      <div
        className="flex w-full transition-transform duration-1000"
        style={{ transform: `translate3d(${-index * transformValue}%, 0, 0)` }}
      >
        {items.map((item, index) => (
          <div
            className="pr-8 flex flex-row w-full md:w-1/2 gap-8 lg:gap-16 shrink-0 items-start"
            key={index}
          >
            {(item.icon as Media).url && (
              <Image
                src={createImageUrl((item.icon as Media).url)}
                alt={(item.icon as Media).alt}
                className="h-16 w-full text-gray-600"
                width="0"
                height="0"
                sizes="100vw"
              />
            )}
            <div>
              <Heading text={item.title} type="h4" as="p" />
              <p dangerouslySetInnerHTML={{ __html: item.description }} className="text-gray-800" />
            </div>
          </div>
        ))}
      </div>

      <div className="mx-auto inline-flex space-x-2 mt-8">
        {items.map((_, idx) => (
          <button
            className="relative h-5 w-5 bg-primary-500 hover:bg-primary-600 transition-colors rounded-full"
            onClick={() => {
              setIndex(idx);
            }}
            aria-label={`Tlačítko položky ${idx + 1}`}
            key={idx}
          >
            {index === idx && (
              <span className="absolute h-2.5 w-2.5 bg-white left-1/2 -translate-x-1/2 top-1/2 -translate-y-1/2 rounded-full" />
            )}
          </button>
        ))}
      </div>
    </div>
  );
};

export default OwlCarousel;
