import escapeHTML from "escape-html";
import Link from "next/link";
import React, { Fragment } from "react";
import YouTube from "react-youtube";
import { BaseText, Text } from "slate";

import { MARGIN_BOTTOM } from "styles/constants";

import { joinClassNames } from "utils/helper";

import Blockquote from "components/Blockquote";
import { Divider } from "components/Divider";
import { Heading } from "components/Heading";
import { Image } from "components/Image";
import { RenderDoc } from "components/RenderDoc";

interface ExtendedBaseText extends BaseText {
  underline: boolean;
  type: string;
}
const serialize = (children) =>
  children?.map((node, i) => {
    if (Text.isText(node as ExtendedBaseText)) {
      let text = <span key={i} dangerouslySetInnerHTML={{ __html: node.text }} />;

      if (node.bold) {
        text = <strong key={i}>{text}</strong>;
      }

      if (node.code) {
        text = <code key={i}>{text}</code>;
      }

      if (node.italic) {
        text = <em key={i}>{text}</em>;
      }

      if (node.underline) {
        text = <u key={i}>{text}</u>;
      }

      // Handle other leaf types here...
      return <Fragment key={i}>{text}</Fragment>;
    }

    if (!node) {
      return null;
    }

    switch (node.type) {
      case "h1":
        return (
          <Heading as="p" type="h1" marginBottom="xs" key={i} text={serialize(node.children)} />
        );
      case "h2":
        return (
          <Heading as="p" type="h2" marginBottom="xs" key={i} text={serialize(node.children)} />
        );
      case "h3":
        return (
          <Heading as="p" type="h3" marginBottom="xs" key={i} text={serialize(node.children)} />
        );
      case "h4":
        return <Heading as="p" type="h4" key={i} text={serialize(node.children)} />;
      case "h5":
        return <Heading as="p" type="h5" key={i} text={serialize(node.children)} />;
      case "h6":
        return <Heading as="p" type="h6" key={i} text={serialize(node.children)} />;
      case "quote":
        return <blockquote key={i}>{serialize(node.children)}</blockquote>;
      case "ul":
        return <ul key={i}>{serialize(node.children)}</ul>;
      case "ol":
        return <ol key={i}>{serialize(node.children)}</ol>;
      case "li":
        return <li key={i}>{serialize(node.children)}</li>;
      case "large-body":
        return (
          <span className="text-lg" key={i}>
            {serialize(node.children)}
          </span>
        );

      case "large-body":
        return (
          <span className="text-lg" key={i}>
            {serialize(node.children)}
          </span>
        );
      case "indent":
        return (
          <div className="ml-6" key={i}>
            {serialize(node.children)}
          </div>
        );
      case "link":
        switch (node.doc.relationTo) {
          case "pages":
            return (
              <Link href={escapeHTML(node.doc.value.slug)} key={i}>
                {serialize(node.children)}
              </Link>
            );
          case "posts":
            return (
              <Link href={`/blog/${escapeHTML(node.doc.value.slug)}`} key={i}>
                {serialize(node.children)}
              </Link>
            );
        }
        break;
      case "video":
        return <YouTube videoId={node.id} key={i} />;
      case "hr":
        return <Divider type="simple" />;
      case "upload":
        return node.relationTo === "documents" ? (
          <RenderDoc data={node.value} key={i} />
        ) : (
          // eslint-disable-next-line jsx-a11y/alt-text
          <Image
            image={node.value}
            width={node.fields?.width}
            showCaption={true}
            caption={serialize(node.fields?.caption)}
            key={i}
          />
        );

      case "blockquote":
        return <Blockquote>{serialize(node.children)}</Blockquote>;

      default:
        return <p key={i}>{serialize(node.children)}</p>;
    }
  });

type Props = {
  richText: any;
  marginBottom?: keyof typeof MARGIN_BOTTOM;
  color?: string;
  prose?: boolean;
};

const RenderContent = ({ richText, marginBottom, color, prose = true }: Props) => {
  return (
    <div
      className={joinClassNames("max-w-none", prose && "prose", MARGIN_BOTTOM[marginBottom])}
      style={{ color: color }}
    >
      {serialize(richText)}
    </div>
  );
};

export default RenderContent;
