import { NextPage } from "next";
import { DefaultSeo } from "next-seo";
import { Bebas_Neue, Montserrat } from "next/font/google";
import Head from "next/head";
import { GoogleAnalytics } from "nextjs-google-analytics";
import {
  ContactDetail,
  Cooky,
  Footer,
  Header,
  Metadatum,
  SocialSite,
  WebsiteSetting,
} from "payload-types";
import {
  fetchContactDetails,
  fetchCookies,
  fetchFooter,
  fetchHeader,
  fetchMetadata,
  fetchSocial,
  fetchWebsiteSettings,
} from "queries/globals";
import { Context, createContext, Fragment } from "react";
import { GoogleReCaptchaProvider } from "react-google-recaptcha-v3";

import "styles/globals.css";

import { createImageUrl } from "utils/helper";

import { CookieConsent } from "components/CookieConsent";

const montserrat = Montserrat({
  subsets: ["latin", "latin-ext"],
  variable: "--font-montserrat",
  display: "swap",
});

const bebasNeue = Bebas_Neue({
  subsets: ["latin", "latin-ext"],
  variable: "--font-bebasneue",
  weight: ["400"],
  display: "swap",
});

type ThemeContextType = {
  socials?: SocialSite;
  header?: Header;
  footer?: Footer;
  metadata?: Metadatum;
  contactDetails?: ContactDetail;
  websiteSettings?: WebsiteSetting;
  cookies?: Cooky;
};

export const GlobalContext: Context<ThemeContextType> = createContext({});

const MyApp: NextPage<any> = ({ Component, pageProps, props }) => {
  const Layout = Component.layout ?? Fragment;
  return (
    <GoogleReCaptchaProvider
      reCaptchaKey={process.env.NEXT_PUBLIC_RECAPTCHA_SITE_KEY}
      scriptProps={{
        async: true,
        defer: true,
        appendTo: "head",
        nonce: undefined,
      }}
    >
      <style jsx global>{`
        :root {
          --font-montserrat: ${montserrat.style.fontFamily};
          --font-bebasneue: ${bebasNeue.style.fontFamily};
        }
      `}</style>
      <GoogleAnalytics trackPageViews defaultConsent="denied" strategy="afterInteractive" />
      <GlobalContext.Provider
        value={{
          socials: props.socials,
          header: props.header,
          footer: props.footer,
          metadata: props.metadata,
          contactDetails: props.contactDetails,
          websiteSettings: props.websiteSettings,
          cookies: props.cookies,
        }}
      >
        <DefaultSeo
          openGraph={{
            type: "website",
            url: "https://energosavings.cz/",
            images: [
              {
                url: createImageUrl(props.metadata?.ogImage.url),
              },
            ],
          }}
          themeColor="#27a79b"
          twitter={{
            handle: props.metadata?.ogCreator,
            site: props.metadata?.ogTwitter,
            cardType: "summary_large_image",
          }}
          defaultTitle={props.metadata?.defaultTitle}
        />
        <div className="font-sans overflow-x-hidden">
          <Layout>
            <Head>
              <meta name="viewport" content="width=device-width, initial-scale=1" />
            </Head>
            <Component {...pageProps} />
            <CookieConsent />
          </Layout>
        </div>
      </GlobalContext.Provider>
    </GoogleReCaptchaProvider>
  );
};

MyApp.getInitialProps = async () => {
  const socials = await fetchSocial();
  const header = await fetchHeader();
  const footer = await fetchFooter();
  const metadata = await fetchMetadata();
  const contactDetails = await fetchContactDetails();
  const websiteSettings = await fetchWebsiteSettings();
  const cookies = await fetchCookies();
  return {
    props: { socials, header, footer, metadata, contactDetails, websiteSettings, cookies },
  };
};

export default MyApp;
