import { BriefcaseIcon, EnvelopeIcon, PhoneIcon } from "@heroicons/react/24/solid";
import { GetStaticProps } from "next";
import { NextSeo } from "next-seo";
import Image from "next/image";
import { Employee, Media } from "payload-types";
import { fetchEmployee, fetchEmployeeSlugs } from "queries/employee";

import { createImageUrl } from "utils/helper";

import { LayoutMain } from "components/LayoutMain";
import { PreviewBar } from "components/PreviewBar";
import { RenderBlocks } from "components/RenderBlocks";

type PageProps = {
  employee: Employee;
  preview: boolean;
};

const EmployeePage = ({ employee, preview }: PageProps) => {
  return (
    <>
      <NextSeo title={employee.name} />
      {preview && <PreviewBar />}
      <div className="max-w-5xl mx-auto px-2 xl:px-0">
        <div className="flex flex-col space-y-4 md:space-y-0 md:flex-row md:space-x-16">
          <Image
            src={
              employee.image
                ? createImageUrl((employee.image as Media).url)
                : "/images/avatar-placeholder.png"
            }
            width={employee.image ? (employee.image as Media).width : 0}
            height={employee.image ? (employee.image as Media).height : 0}
            alt={employee.name}
            className="h-24 w-24 md:h-32 md:w-32 border-2 border-primary-600 p-1 rounded-full object-cover"
          />
          <div className="mb-8">
            <div className="mb-8">
              <p className="font-display text-5xl text-primary-600 mb-4">{employee.name}</p>
              <p className="flex text-gray-600 mb-2">
                <BriefcaseIcon className="w-4 mr-2" /> {employee.position}
              </p>
              <a
                href={`mailto:${employee.email}`}
                className="flex items-center text-gray-600 hover:text-primary-600 transition-colors mb-2"
              >
                <EnvelopeIcon className="w-4 mr-2" />
                {employee.email}
              </a>
              {employee.telephoneNumber && (
                <a
                  href={`tel:${employee.telephoneNumber}`}
                  className="flex items-center text-gray-600 hover:text-primary-600 transition-colors"
                >
                  <PhoneIcon className="w-4 mr-2" />
                  {employee.telephoneNumber}
                </a>
              )}
            </div>
            <RenderBlocks blocks={employee.layout} />
          </div>
        </div>
      </div>
      <div></div>
    </>
  );
};

export const getStaticProps: GetStaticProps<{ employee: Employee }> = async ({
  params,
  preview = false,
}) => {
  const employee: Employee = await fetchEmployee(params.slug as string, preview);
  if (!employee) {
    return {
      notFound: true,
    };
  }

  return {
    props: { employee, preview },
  };
};

export const getStaticPaths = async () => {
  const slugs = await fetchEmployeeSlugs();
  return {
    paths: slugs?.map((slug: string) => `/employee/${slug}`),
    fallback: "blocking",
  };
};

EmployeePage.layout = LayoutMain;
export default EmployeePage;
