import { Container } from "../blocks/Container";
import { slugField } from "../fields/slug";
import { formatPreviewURL } from "../utilities/formatPreviewUrl";
import { regeneratePage } from "../utilities/regeneratePage";
import type { CollectionConfig } from "payload/types";

export const Posts: CollectionConfig = {
  slug: "posts",
  admin: {
    useAsTitle: "title",
    preview: (doc) => formatPreviewURL("posts", doc),
  },
  versions: {
    drafts: true,
  },
  access: {
    read: () => {
      return true;
    },
  },
  hooks: {
    afterChange: [
      ({ req: { payload } }) => {
        regeneratePage({
          payload,
        });
      },
    ],
    afterDelete: [
      ({ req: { payload } }) => {
        regeneratePage({
          payload,
        });
      },
    ],
  },
  labels: {
    singular: "Článek",
    plural: "Články",
  },
  fields: [
    {
      name: "title",
      label: "Název článku",
      type: "text",
      required: true,
    },
    {
      name: "image",
      label: "Náhledový obrázek článku",
      type: "upload",
      relationTo: "media",
      required: true,
    },
    {
      name: "excerpt",
      label: "Shrnutí článku",
      type: "textarea",
      required: true,
    },
    {
      name: "category",
      label: "Kategorie",
      type: "relationship",
      relationTo: "categories",
      hasMany: true,
    },
    {
      name: "content",
      label: "Komponenty",
      labels: {
        singular: "Komponenta",
        plural: "Komponenty",
      },
      type: "blocks",
      blocks: [Container],
      required: true,
    },
    slugField("title", { required: true }),
    {
      name: "author",
      type: "relationship",
      relationTo: "users",
      maxDepth: 10,
      required: true,
      admin: {
        position: "sidebar",
      },
    },
    {
      name: "publishDate",
      type: "date",
      admin: {
        position: "sidebar",
        description: "Posts will not be public until this date",
      },
      defaultValue: () => new Date(),
    },
  ],
  endpoints: [
    {
      path: "/slugs",
      method: "get",
      handler: async (req, res, next) => {
        const { docs } = await req.payload.find({
          collection: "posts",
          pagination: false,
        });
        const slugs = docs.map((post) => post.slug);
        if (docs) {
          res.status(200).send(slugs);
        } else {
          res.status(404).send({ error: "not found" });
        }
      },
    },
    {
      path: "/all",
      method: "get",
      handler: async (req, res) => {
        const { docs } = await req.payload.find({
          collection: "posts",
          pagination: false,
        });
        if (docs) {
          res.status(200).send(docs);
        } else {
          res.status(404).send({ error: "not found" });
        }
      },
    },
  ],
};
