import { validateHexColor } from "./config";
import "./styles.scss";
import { Button } from "payload/components";
import { Props } from "payload/components/fields/Text";
import { useFieldType } from "payload/components/forms";
import { Label } from "payload/components/forms";
import { usePreferences } from "payload/components/preferences";
import React, { useEffect, useState, useCallback, Fragment } from "react";

const defaultColors = [
  "#f1fcf9",
  "#d1f6ef",
  "#6edacb",
  "#27a79b",
  "#1b6a65",
  "#B2FFD6",
  "#1a4745",
  "#fdf9e9",
  "#faf1c7",
  "#f2c952",
  "#dc9916",
  "#975311",
  "#6b3718",
  "#1a4745",
  "#ffffff",
  "#f1f5f9",
  "#cbd5e1",
  "#64748b",
  "#334155",
  "#0f172a",
  "#000000",
  "#ef4444",
  "#f97316",
  "#facc15",
  "#22c55e",
  "#06b6d4",
  "#0ea5e9",
  "#3b82f6",
  "#6366f1",
  "#ec4899",
];
const baseClass = "custom-color-picker";

const preferenceKey = "color-picker-colors";

const InputField: React.FC<Props> = (props) => {
  const { path, label, required } = props;

  const { value = "", setValue } = useFieldType({
    path,
    validate: validateHexColor,
  });

  const { getPreference, setPreference } = usePreferences();
  const [colorOptions, setColorOptions] = useState(defaultColors);
  const [isAdding, setIsAdding] = useState(false);
  const [colorToAdd, setColorToAdd] = useState("");

  useEffect(() => {
    const mergeColorsFromPreferences = async () => {
      const colorPreferences = await getPreference<string[]>(preferenceKey);
      if (colorPreferences) {
        setColorOptions(colorPreferences);
      }
    };
    mergeColorsFromPreferences();
  }, [getPreference, setColorOptions]);

  const handleAddColor = useCallback(() => {
    setIsAdding(false);
    setValue(colorToAdd);

    if (colorOptions.indexOf(colorToAdd) > -1) return;

    let newOptions = colorOptions;
    newOptions.unshift(colorToAdd);

    setColorOptions(newOptions);
    setPreference(preferenceKey, newOptions);
  }, [colorOptions, setPreference, colorToAdd, setIsAdding, setValue]);

  return (
    <div className={baseClass}>
      <Label htmlFor={path} label={label} required={required} />
      {isAdding && (
        <div>
          <input
            className={`${baseClass}__input`}
            type="text"
            placeholder="#000000"
            onChange={(e) => setColorToAdd(e.target.value)}
            value={colorToAdd}
          />
          <Button
            className={`${baseClass}__btn`}
            buttonStyle="primary"
            iconPosition="left"
            iconStyle="with-border"
            size="small"
            onClick={handleAddColor}
            disabled={validateHexColor(colorToAdd) !== true}
          >
            Add
          </Button>
          <Button
            className={`${baseClass}__btn`}
            buttonStyle="secondary"
            iconPosition="left"
            iconStyle="with-border"
            size="small"
            onClick={() => setIsAdding(false)}
          >
            Cancel
          </Button>
        </div>
      )}
      {!isAdding && (
        <Fragment>
          <ul className={`${baseClass}__colors`}>
            {colorOptions.map((color, i) => (
              <li key={i}>
                <button
                  type="button"
                  key={color}
                  className={`chip ${
                    color === value ? "chip--selected" : ""
                  } chip--clickable`}
                  style={{ backgroundColor: color }}
                  aria-label={color}
                  onClick={() => setValue(color === value ? "" : color)}
                />
              </li>
            ))}
          </ul>
          <Button
            className="add-color"
            icon="plus"
            buttonStyle="icon-label"
            iconPosition="left"
            iconStyle="with-border"
            onClick={() => {
              setIsAdding(true);
              setValue("");
            }}
          />
        </Fragment>
      )}
    </div>
  );
};
export default InputField;
